import Result from './result';
import { TargetFunc, TestCallbacks } from './callbacks';
declare namespace Benchmark {
    type BeforeFunc = (
    /**
     * Callback function for setup before execute {@link Benchmark}.
     *
     * @return  {@link Benchmark} will await if returns Promise.
     *
     * @since 1.0.0
     */
    () => Promise<void> | void);
    type BeforeEachFunc = (
    /**
     * Callback function for setup before each execute.
     *
     * @param count  Count of done tests in this benchmark.
     *
     * @return  {@link Benchmark} will await if returns Promise.
     *
     * @since 1.0.0
     */
    (count: number) => Promise<void> | void);
    type AfterEachFunc = (
    /**
     * Callback function for teardown each execute.
     *
     * @param count  Count of done tests in this benchmark.
     * @param msec   Duration of this execution.
     *
     * @return  {@link Benchmark} will await if returns Promise.
     *
     * @since 1.0.0
     */
    (count: number, msec: number) => Promise<void> | void);
    type AfterFunc = (
    /**
     * Callback function for teardown after {@link Benchmark}
     *
     * @param result  Result of this benchmark.
     *
     * @return  {@link Benchmark} will await if returns Promise.
     *
     * @since 1.0.0
     */
    (result: Result) => Promise<void> | void);
}
/**
 * The options for {@link Benchmark}.
 *
 * @since 0.3.4
 */
export declare type BenchmarkOptions = {
    /**
     * @internal
     */
    __proto__?: BenchmarkOptions;
    /**
     * Name of this benchmark.
     */
    name?: string;
    /**
     * Wanted maximum error rate. See {@link Benchmark.targetErrorRate}.
     */
    targetErrorRate?: number;
    /**
     * Maximum number of executing test. See {@link Benchmark.maxNumber}.
     */
    maxNumber?: number;
    /**
     * Minimal number of executing test. See {@link Benchmark.minNumber}.
     */
    minNumber?: number;
    /**
     * the number of executing the test. see {@link Benchmark.number}.
     */
    number?: number;
    /**
     * Setup function. See {@link Benchmark.before}.
     */
    before?: Benchmark.BeforeFunc;
    /**
     * Setup function. See {@link Benchmark.beforeEach}.
     */
    beforeEach?: Benchmark.BeforeEachFunc;
    /**
     * Target function for benchmarking. See {@link Benchmark.fun}.
     */
    fun?: TargetFunc;
    /**
     * Teardown function. See {@link Benchmark.afterEach}.
     */
    afterEach?: Benchmark.AfterEachFunc;
    /**
     * Teardown function. See {@link Benchmark.after}.
     */
    after?: Benchmark.AfterFunc;
};
/**
 * Class for benchmarking.
 *
 * Benchmark will execute by flow like this.
 *
 * - {@link Benchmark.before | before}
 * - {@link Benchmark.beforeEach | beforeEach}
 * - {@link Benchmark.fun | fun}
 * - {@link Benchmark.afterEach | afterEach}
 * - {@link Benchmark.after | after}
 *
 * Each function can override with options of the constructor.
 *
 *
 * ``` typescript
 * import Benchmark from 'asyncmark';
 *
 *
 * new Benchmark({
 *     name: 'timeout',
 *     fun() {
 *         return new Promise((resolve, reject) => {
 *             setTimeout(resolve, 100);
 *         });
 *     },
 * }).run().catch(console.error);
 * ```
 */
export default class Benchmark {
    /**
     * Name of this benchmark.
     */
    readonly name: string;
    /**
     * Wanted maximum error rate.
     * This value will be ignore if set {@link Benchmark.number}.
     */
    readonly targetErrorRate: number;
    /**
     * Maximum number of executing test.
     * This value will be ignore if set {@link Benchmark.number}.
     */
    readonly maxNumber: number;
    /**
     * Minimal number of executing test.
     * This value will be ignore if set {@link Benchmark.number}.
     */
    readonly minNumber: number;
    /**
     * The number of executing the test.
     * Will decide automatically in between {@link Benchmark.minNumber} to {@link Benchmark.maxNumber}
     * if set null.
     */
    readonly number: number;
    /**
     * Setup before execute benchmark.
     *
     * At the time executing this method, `this` is the unique object for the benchmark.
     * So you can use `this` for storing testing data like a database.
     * Data of `this` that set in this method will discard after call {@link Benchmark.after}
     *
     * In default, do nothing.
     */
    before: Benchmark.BeforeFunc;
    /**
     * Setup before each tests.
     *
     * At the time executing this method, `this` is the unique object for the test.
     * So you can use `this` for storing testing data.
     * Data of `this` that set in this method will discard after call {@link Benchmark.afterEach}
     *
     * In default, do nothing.
     */
    beforeEach: Benchmark.BeforeEachFunc;
    /**
     * The target function for benchmarking.
     *
     * At the time executing this method, `this` is the unique object for the test.
     * So you can use `this` for storing testing data.
     * Data of `this` that set in this method will discard after call {@link Benchmark.afterEach}
     *
     * In default, couses error that `Error('target function is not defined')`.
     */
    fun: TargetFunc;
    /**
     * Teardown after each tests.
     *
     * At the time executing this method, `this` is the unique object for the test.
     * So you can use `this` for storing testing data.
     * Data of `this` that set in this method will discard after call this method.
     *
     * In default, do nothing.
     */
    afterEach: Benchmark.AfterEachFunc;
    /**
     * Teardown after execute benchmark.
     *
     * At the time executing this method, `this` is the unique object for the benchmark.
     * So you can use `this` for storing testing data like a database.
     * Data of `this` that set in this method will discard after call this method.
     *
     * In default, shows test result.
     */
    after: Benchmark.AfterFunc;
    /**
     * @param options  The options for this benchmark or benchmarking function.
     */
    constructor(options: BenchmarkOptions | TargetFunc);
    /**
     * Execute benchmark.
     *
     * @param context    The `this` for each benchmarking functions.
     *                   `__proto__` will override with this instance.
     * @param callbacks  Callback functions.
     *
     * @return A result of benchmark.
     */
    run(context?: any, { beforeTest, afterTest }?: TestCallbacks): Promise<Result>;
}
export {};
