/**
 * The result of benchmark.
 *
 * This value will included outlier. Please use {@link Result.dropOutlier} if you want drop they.
 */
export default class Result {
    /**
     * Name of this test.
     */
    readonly name: string;
    /**
     * Times of benchmark result in milliseconds.
     */
    readonly msecs: number[];
    /**
     * @param name   Name of benchmark.
     * @param msecs  Times of benchmark result.
     */
    constructor(name: string, msecs: number[]);
    /**
     * Total milliseconds of this benchmark.
     */
    get total(): number;
    /**
     * The time of fastest test in milliseconds.
     */
    get fastest(): number;
    /**
     * The time of slowest test in milliseconds.
     */
    get slowest(): number;
    /**
     * Average time of this benchmark in milliseconds.
     */
    get average(): number;
    /**
     * Time unbiased sample variance of times.
     */
    get variance(): number;
    /**
     * Standard division of times.
     */
    get std(): number;
    /**
     * Standard error of the mean of times.
     */
    get sem(): number;
    /**
     * Guessed error range of this benchmark.
     */
    get errorRange(): number;
    /**
     * Error range per average time.
     */
    get errorRate(): number;
    /**
     * Operations per seconds.
     */
    get opsPerSec(): number;
    /**
     * Make new Result that droped outlier.
     *
     * @param threshold  The threshold of outlier testing.
     *
     * @return  New {@link Result} instance.
     */
    dropOutlier(threshold?: number): Result;
    /**
     * Convert to string for printing.
     *
     * @return  Human redable string
     */
    toString(): string;
    /**
     * Assertion if it taked more (or less) time than expected.
     *
     * Expected rule format is `{operator}{number}{unit}`; use like `<=10msec`.
     * Operator and unit are can omit. If omitted, uses `<=` and `msec`.
     *
     * This function will throw {@link AsyncMarkAssertionError} if result is unaccepcable.
     *
     * @param expected  Expected time in milliseconds number or string value.
     *                  e.g. `<10ms` or `>=20s`.
     *
     * @return
     *
     * ## Supported operators
     * |example       |means              |
     * |--------------|-------------------|
     * |"<42"         |faster than 42 msec|
     * |"<=42" or omit|42 msec or faster  |
     * |">42"         |slower than 42 msec|
     * |">=42"        |42 msec or slower  |
     *
     * ## Supported units
     * |example           |means       |
     * |------------------|------------|
     * |"42s" or "42sec"  |seconds     |
     * |"42ms" or "42msec"|milliseconds|
     * |"42us" or "42usec"|microseconds|
     * |"42ns" or "42nsec"|nanoseconds |
     *
     * ## Examples
     * ### Simple usage
     *
     * ``` typescript
     * const result = await Benchmark(function() {
     *     # do something that expect done in least 100msec
     * }).run();
     *
     * result.assert(100);
     * ```
     *
     * ### Combination rule
     *
     * ``` typescript
     * const result = await Benchmark(async function() {
     *     await sleep_function(100);
     * }).run();
     *
     * result.assert('>90ms', '<110ms');
     * ```
     *
     * @since 0.3.0
     */
    assert(...expected: (number | string)[]): void;
}
